#========================================================================================
# package      DE_EPAGES::Amazon::API::Constants
# state        public
#----------------------------------------------------------------------------------------
# description  This is the place for common Amazon values and constants.
#========================================================================================
package DE_EPAGES::Amazon::API::Constants;
use base Exporter;

use strict;
use Readonly;

our @EXPORT_OK = qw(
  $OAUTH_STATE_HASH_SECRET
  $OAUTH_TOKEN_ENDPOINT

  %AMAZON_ITEM_CONDITIONS
  %AMAZON_ONLINESTATUS
  %AMAZON_PROCESSINGSTATUS
  %AMAZON_FEEDS
  %AMAZON_PENDING_FEEDS
  MERCHANT_SOAP_URI
  SP_API_REPORTS_RESSOURCE
  SP_API_SELLERS_RESSOURCE
  ItemConditionValues
  %MAP_LOCALE
  MIN_DELIVERY_LATENCY
  MAX_DELIVERY_LATENCY
  REPORT_SCHEDULER_PERIOD
);


#-------------------
#---------------
#-- SP-API
Readonly::Scalar our $OAUTH_STATE_HASH_SECRET => 'Akd1394KADJ19fd9f1039Zkal139aJD3';

Readonly::Scalar our $OAUTH_TOKEN_ENDPOINT => 'https://api.amazon.com/auth/o2/token';
#--
#---------------
#-------------------

our %AMAZON_ITEM_CONDITIONS = (
    'New'                   => 0,
    'Refurbished'           => 1,
    'UsedLikeNew'           => 2,
    'UsedVeryGood'          => 3,
    'UsedGood'              => 4,
    'UsedAcceptable'        => 5,
    'CollectibleLikeNew'    => 6,
    'CollectibleVeryGood'   => 7,
    'CollectibleGood'       => 8,
    'CollectibleAcceptable' => 9,
);

our %AMAZON_ONLINESTATUS = (
    'OFFER_OFFLINE'    => 0,   # preparing, not online
    'OFFER_FAILED'     => 1,   # upload error
    'OFFER_PROCESSING' => 2,   # pending, not online yet (set to OFFER_ONLINE by scheduler)
    'OFFER_ONLINE'     => 4,   # online
    'OFFER_HIDDEN'     => 8,   # deactivated, still online (set to OFFER_OFFLINE by scheduler)
);

our %AMAZON_PROCESSINGSTATUS = (
    '_IN_PROGRESS_' => 0,
    '_DONE_'        => 1,
);

our %AMAZON_FEEDS = (
    '_POST_PRODUCT_DATA_'                => 0,
    '_POST_PRODUCT_RELATIONSHIP_DATA_'   => 1,
    '_POST_PRODUCT_OVERRIDES_DATA_'      => 2,
    '_POST_PRODUCT_IMAGE_DATA_'          => 3,
    '_POST_PRODUCT_PRICING_DATA_'        => 4,
    '_POST_INVENTORY_AVAILABILITY_DATA_' => 5,
    '_POST_ORDER_ACKNOWLEDGEMENT_DATA_'  => 6,
    '_POST_ORDER_FULFILLMENT_DATA_'      => 7,
    '_POST_PAYMENT_ADJUSTMENT_DATA_'     => 8,
    '_POST_STORE_DATA_'                  => 9,
    '_POST_OFFER_ONLY_DATA_'             => 10,
);

our %AMAZON_PENDING_FEEDS = (
    'orders'   => 'GET_ORDER_REPORT_DATA_INVOICING'
);

use constant MERCHANT_SOAP_URI => 'http://www.amazon.com/merchants/merchant-interface/MerchantInterface';
use constant SP_API_REPORTS_RESSOURCE => '/reports/2021-06-30';
use constant SP_API_SELLERS_RESSOURCE => '/sellers/v1';

our %MAP_LOCALE = (
    'fr' => 'fr_FR',
    'de' => 'de_DE',
    'it' => 'it_IT',
    'es' => 'es_ES',
    'uk' => 'en_GB',
);


# allowed range in days for the delivery latency
use constant MIN_DELIVERY_LATENCY => 1;
use constant MAX_DELIVERY_LATENCY => 30;

use constant REPORT_SCHEDULER_PERIOD => 'PT15M';
#========================================================================================
# function     ItemConditionValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       %hSplitCond = DE_EPAGES::Amazon::API::Constants->ItemConditionValues("UsedLikeNew");
#----------------------------------------------------------------------------------------
# description  Splits the condition value into a hash
#----------------------------------------------------------------------------------------
# input        $Condition  | combined condition string | string
# return       %hSplitCond | split condition values    | hash
#               used keys are:
#               <ul>
#                 <li>CompleteString - input as-is
#                 <li>MainCondition  - main condition, e.g. "Used"
#                 <li>SubCondition   - subcondition, e.g. "LikeNew"
#               </ul>
#========================================================================================
sub ItemConditionValues ($) {
    my ($ID) = @_;
    my $hRet = {};
    foreach my $key ( keys %AMAZON_ITEM_CONDITIONS ) {
        my $value = $AMAZON_ITEM_CONDITIONS{$key};
        if (   $value == $ID
            && $key =~ /^(New|Used|Collectible|Refurbished)(.*)/x )
        {
            $hRet = {
                'CompleteString' => $key,
                'MainCondition'  => $1,
                'SubCondition'   => ( $2 ? $2 : $1 ),
            };
            return $hRet;
        }
    }
    return $hRet;
}

1;
