#========================================================================================
# package      DE_EPAGES::Spreedly::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UI::Shop
#========================================================================================
package DE_EPAGES::Spreedly::t::unit::UI_Shop_SpreedlyPaymentCallback;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;
use Test::Exception;
use strict;

use DE_EPAGES::Spreedly::API::Constants qw (
    PURCHASE_RESPONSE_STATE_PROCESSING
    PURCHASE_RESPONSE_STATE_SUCEEDED
    PAYMENT_STATUS_PROCESSING
    PAYMENT_STATUS_FAILED
    PAYMENT_STATUS_PAID );

sub test {
    my $self = shift;

    $self->_prepareMocks();

    $self->test_SpreedlyPaymentCallback_status_processing_to_paid();
    $self->test_SpreedlyPaymentCallback_status_succeed_to_failed();
    $self->test_SpreedlyPaymentCallback_status_processing_to_processing();
    $self->test_SpreedlyPaymentCallback_status_processing_to_failed();
    $self->test_SpreedlyPaymentCallback_status_failed_to_paid();

    done_testing();
}

sub test_SpreedlyPaymentCallback_status_processing_to_paid {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $Servlet = $self->_buildServlet({
        'Shop' => $self->memory->get('MultistepShop'),
        'IsMobile' => 1,
        'PaymentLineItemGUID' => '445566-332211-221144',
        'ObjectID' => '12233',
        'DereferrerToken' => 'xGHPdGpZ',
    });

    my $SUT = $self->_buildSUT({
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop'),
        'isPaymentSuccessful' => 0,
        'getPaymentStatusFromTransactionState' => PAYMENT_STATUS_PAID,
        'getTransactionStatus' => PAYMENT_STATUS_PROCESSING,
        '_requestSpreedlyTransactionByToken' => {
            'successful'    => '1',
            'transactionNo' => '123434',
            'state' => PURCHASE_RESPONSE_STATE_SUCEEDED
        }
    });

    ## WHEN
    $SUT->SpreedlyPaymentCallback($Servlet);

    my $LineItemPaymentSpreedly = $self->memory->get('LineItemPayment');

    ## THEN
    is( $LineItemPaymentSpreedly->call_pos( 3 ), 'isPaymentSuccessful', "$SubTestName - isPaymentSuccessful has been called." );

    is( $LineItemPaymentSpreedly->call_pos( 4 ), 'getPaymentStatusFromTransactionState', "$SubTestName - getPaymentStatusFromTransactionState has been called." );
    is( ($LineItemPaymentSpreedly->call_args( 4 ))[1], PURCHASE_RESPONSE_STATE_SUCEEDED, "$SubTestName - getPaymentStatusFromTransactionState has been called with correct arguments." );

    is( $LineItemPaymentSpreedly->call_pos( 5 ), 'updateTransactionStatus', "$SubTestName - updateTransactionStatus has been called." );
    is( ($LineItemPaymentSpreedly->call_args( 5 ))[1], PAYMENT_STATUS_PAID, "$SubTestName - updateTransactionStatus has been called with correct arguments." );

    return;
}

sub test_SpreedlyPaymentCallback_status_succeed_to_failed {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $Servlet = $self->_buildServlet({
        'Shop' => $self->memory->get('MultistepShop'),
        'IsMobile' => 1,
        'PaymentLineItemGUID' => '445566-332211-221144',
        'ObjectID' => '12233',
        'DereferrerToken' => 'xGHPdGpZ',
    });

    my $SUT = $self->_buildSUT({
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop'),
        'isPaymentSuccessful' => 1,
        'getPaymentStatusFromTransactionState' => PAYMENT_STATUS_FAILED,
        'getTransactionStatus' => PAYMENT_STATUS_PAID,
        '_requestSpreedlyTransactionByToken' => {
            'successful'    => '1',
            'transactionNo' => '123434',
            'state' => 'failed'
        }
    });

    ## WHEN
    $SUT->SpreedlyPaymentCallback($Servlet);

    my $LineItemPaymentSpreedly = $self->memory->get('LineItemPayment');

    ## THEN
    is( $LineItemPaymentSpreedly->call_pos( 3 ), 'isPaymentSuccessful', "$SubTestName - isPaymentSuccessful has been called." );

    is( $LineItemPaymentSpreedly->call_pos( 4 ), undef, "$SubTestName - getPaymentStatusFromTransactionState has not been called." );
    is( $LineItemPaymentSpreedly->call_pos( 5 ), undef, "$SubTestName - updateTransactionStatus has not been called." );
    is( $LineItemPaymentSpreedly->call_pos( 6 ), undef, "$SubTestName - setOrderToPaid has not been called." );

    return;
}

sub test_SpreedlyPaymentCallback_status_failed_to_paid {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $Servlet = $self->_buildServlet({
        'Shop' => $self->memory->get('MultistepShop'),
        'IsMobile' => 1,
        'PaymentLineItemGUID' => '445566-332211-221144',
        'ObjectID' => '12233',
        'DereferrerToken' => 'xGHPdGpZ',
    });

    my $SUT = $self->_buildSUT({
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop'),
        'isPaymentSuccessful' => 0,
        'getPaymentStatusFromTransactionState' => PAYMENT_STATUS_PAID,
        'getTransactionStatus' => PAYMENT_STATUS_FAILED,
        '_requestSpreedlyTransactionByToken' => {
            'successful'    => '1',
            'transactionNo' => '123434',
            'state' => PURCHASE_RESPONSE_STATE_SUCEEDED
        }
    });

    ## WHEN
    $SUT->SpreedlyPaymentCallback($Servlet);

    my $LineItemPaymentSpreedly = $self->memory->get('LineItemPayment');

    ## THEN
    is( $LineItemPaymentSpreedly->call_pos( 3 ), 'isPaymentSuccessful', "$SubTestName - isPaymentSuccessful has been called." );

    is( $LineItemPaymentSpreedly->call_pos( 4 ), undef, "$SubTestName - getPaymentStatusFromTransactionState has not been called." );
    is( $LineItemPaymentSpreedly->call_pos( 5 ), undef, "$SubTestName - updateTransactionStatus has not been called." );
    is( $LineItemPaymentSpreedly->call_pos( 6 ), undef, "$SubTestName - setOrderToPaid has not been called." );

    return;
}

sub test_SpreedlyPaymentCallback_status_processing_to_processing {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $Servlet = $self->_buildServlet({
        'Shop' => $self->memory->get('MultistepShop'),
        'IsMobile' => 1,
        'PaymentLineItemGUID' => '445566-332211-221144',
        'ObjectID' => '12233',
        'DereferrerToken' => 'xGHPdGpZ',
    });

    my $SUT = $self->_buildSUT({
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop'),
        'isPaymentSuccessful' => 0,
        'getPaymentStatusFromTransactionState' => PAYMENT_STATUS_PROCESSING,
        'getTransactionStatus' => PAYMENT_STATUS_PROCESSING,
        '_requestSpreedlyTransactionByToken' => {
            'successful'    => '1',
            'transactionNo' => '123434',
            'state' => PURCHASE_RESPONSE_STATE_PROCESSING
        }
    });

    ## WHEN
    $SUT->SpreedlyPaymentCallback($Servlet);

    my $LineItemPaymentSpreedly = $self->memory->get('LineItemPayment');

    ## THEN
    is( $LineItemPaymentSpreedly->call_pos( 3 ), 'isPaymentSuccessful', "$SubTestName - isPaymentSuccessful has been called." );

    is( $LineItemPaymentSpreedly->call_pos( 4 ), 'getPaymentStatusFromTransactionState', "$SubTestName - getPaymentStatusFromTransactionState has been called." );
    is( ($LineItemPaymentSpreedly->call_args( 4 ))[1], PURCHASE_RESPONSE_STATE_PROCESSING, "$SubTestName - getPaymentStatusFromTransactionState has been called with correct arguments." );

    is( $LineItemPaymentSpreedly->call_pos( 5 ), undef, "$SubTestName - updateTransactionStatus has not been called." );
    is( $LineItemPaymentSpreedly->call_pos( 6 ), undef, "$SubTestName - setOrderToPaid has not been called." );

    return;
}

sub test_SpreedlyPaymentCallback_status_processing_to_failed {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $Servlet = $self->_buildServlet({
        'Shop' => $self->memory->get('MultistepShop'),
        'IsMobile' => 1,
        'PaymentLineItemGUID' => '445566-332211-221144',
        'ObjectID' => '12233',
        'DereferrerToken' => 'xGHPdGpZ',
    });

    my $SUT = $self->_buildSUT({
        'ID' => '20304',
        'Shop' => $self->memory->get('MultistepShop'),
        'isPaymentSuccessful' => 0,
        'getPaymentStatusFromTransactionState' => PAYMENT_STATUS_FAILED,
        'getTransactionStatus' => PAYMENT_STATUS_PROCESSING,
        '_requestSpreedlyTransactionByToken' => {
            'successful'    => '1',
            'transactionNo' => '123434',
            'state' => 'failed'
        }
    });

    ## WHEN
    $SUT->SpreedlyPaymentCallback($Servlet);

    my $LineItemPaymentSpreedly = $self->memory->get('LineItemPayment');

    ## THEN
    is( $LineItemPaymentSpreedly->call_pos( 3 ), 'isPaymentSuccessful', "$SubTestName - isPaymentSuccessful has been called." );

    is( $LineItemPaymentSpreedly->call_pos( 4 ), 'getPaymentStatusFromTransactionState', "$SubTestName - getPaymentStatusFromTransactionState has been called." );
    is( ($LineItemPaymentSpreedly->call_args( 4 ))[1], 'failed', "$SubTestName - getPaymentStatusFromTransactionState has been called with correct arguments." );

    is( $LineItemPaymentSpreedly->call_pos( 5 ), 'updateTransactionStatus', "$SubTestName - updateTransactionStatus has been called." );
    is( ($LineItemPaymentSpreedly->call_args( 5 ))[1], PAYMENT_STATUS_FAILED, "$SubTestName - getPaymentStatusFromTransactionState has been called with correct arguments." );

    return;
}

sub _prepareMocks {
    my $self = shift;

    my $UnityShop = $self->mockFactory->buildShop({
        'Alias' => 'UnityShop',
        'ID' => '123',
        'IsUnityRegistered' => 1
    });
    my $MultistepShop = $self->mockFactory->buildShop({
       'Alias' => 'MultistepShop',
       'ID' => '123',
       'IsUnityRegistered' => 0,
       'OrderProcess' => 0
    });

    my $PaymentMethodOrderAfter = $self->mockFactory->build(
        'DE_EPAGES::Payment::API::Object::PaymentMethod', {
            'DoPaymentOnBasket' => 1
        }
    );

    $self->memory->add('UnityShop', $UnityShop->getMockObject());
    $self->memory->add('MultistepShop', $MultistepShop->getMockObject());
    $self->memory->add('PaymentMethodOrderAfter', $PaymentMethodOrderAfter->getMockObject());
    $self->memory->add('Basket', undef);
    $self->memory->add('LineItemPayment', undef);

    return;
}

sub _buildSUT {
    my $self = shift;
    my ($hParams) = @_;

    my $LineItemPayment = $self->_createLineItemPayment($hParams);

    $self->memory->replace('LineItemPayment', $LineItemPayment);

    my $SUT = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Spreedly::UI::Shop', []);
    $SUT->mock('_LogPayment', undef, ['string', 'string', 'hashref']);
    $SUT->mock('_ValidateDereferrerToken', 1, ['string', 'string']);
    $SUT->mock('BlockMonitor', undef, ['object']);
    $SUT->mock('_LoadObjectByGUID', $LineItemPayment, ['string']);
    $SUT->mock('_requestSpreedlyTransactionByToken', $hParams->{'_requestSpreedlyTransactionByToken'}, ['object', 'string', 'int']);

    return $SUT->getMockObject();
}

sub _buildServlet {
    my $self = shift;
    my ($hParams) = @_;

    my $Servlet = DE_EPAGES::Test::Mock::MockifyServlet->new($hParams->{'Shop'});

    $Servlet->addVars({
        'IsMobile' => $hParams->{'IsMobile'},
    });

    $Servlet->addParams({
        'PaymentLineItemGUID' => $hParams->{'PaymentLineItemGUID'},
        'ObjectID' => $hParams->{'ObjectID'},
        'DereferrerToken' => $hParams->{'DereferrerToken'}
    });

    my $MockedServlet = $Servlet->getMockObject();
    $MockedServlet->{'Response'} = HTTP::Response->new( 200 );

    return $MockedServlet;
}

sub _createLineItemPayment {
    my $self = shift;
    my ($hAttributes) = @_;

    my $MockedLineItemContainer = $self->mockFactory->buildLineItemContainer({
        'Parent' => $self->_buildBasket({}),
        'instanceOf' => 'LineItemContainer',
    });

    my $LineItemContainer = $MockedLineItemContainer->getMockObject();

    my $hLineItemPaymentAttributes = {
        'ID' => 123456,
        'Parent' => $LineItemContainer,
        'instanceOf' => 'LineItemPaymentSpreedly',
        'PaymentMethod' => $self->_createPaymentMethod(),
        'Alias' => 'SpreedlyTest',
    };

    my $MockedLineItemPayment = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Spreedly::API::Object::LineItemPaymentSpreedly', $hLineItemPaymentAttributes );

    $MockedLineItemPayment->mock(
        'getTransactionToken',
        sub {
            return 123456;
        }
    );

    $MockedLineItemPayment->mock(
        'isPaymentSuccessful',
        sub {
            return $hAttributes->{'isPaymentSuccessful'};
        }
    );

    $MockedLineItemPayment->mock(
        'getPaymentStatusFromTransactionState',
        sub {
            return $hAttributes->{'getPaymentStatusFromTransactionState'};
        }
    );

    $MockedLineItemPayment->mock(
        'getTransactionStatus',
        sub {
            return $hAttributes->{'getTransactionStatus'};
        }
    );

    $MockedLineItemPayment->mock(
        'updateTransactionStatus',
        sub {
            return;
        }
    );

    $MockedLineItemPayment->mock(
        'setOrderToPaid',
        sub {
            return;
        }
    );

    return $MockedLineItemPayment;
}

sub _createPaymentMethod {
    my $self = shift;
    my ($hAttributes) = @_;

    my $hPaymentMethodAttributes = {
        'PaymentType' => $self->_createPaymentType($hAttributes),
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Payment::API::Object::PaymentMethod',
        $hPaymentMethodAttributes
    );

    return $Mockify->getMockObject();
}

sub _createPaymentType {
    my $self = shift;
    my ($hAttributes) = @_;

    my $hPaymentTypeAttributes = {
        'Alias' => 'MyShop',
        'TestAccountName' => 'ca_82tCJZgOteUhVj91ulpAR7TFF8m1UasG',
        'GatewayName' => 'test',
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Payment::API::Object::PaymentType',
        $hPaymentTypeAttributes
    );

    return $Mockify->getMockObject();
}

sub _buildBasket {
    my $self = shift;
    my ($hAttributes) = @_;

    my $Basket = $self->mockFactory->buildBasket({
        'Shop' => $hAttributes->{'Shop'},
        'Site' => $hAttributes->{'Shop'},
        'ID' => $hAttributes->{'ID'},
        'WebUrlUnitySSL' => 'https://unity-instance.de:7000',
        'User' => 'Max Mustermann',
        'instanceOf' => 'Basket',
    });
    $self->memory->replace('Basket', $Basket->getMockObject());
    return $self->memory->get('Basket');
}

__PACKAGE__->runScript();
